use crate::commands::{Command, RequestType};
use crate::context::Context;
use crate::Result;
use geoffrey_db::helper::{find_location_by_name, load_location};
use geoffrey_models::models::locations::Location;
use geoffrey_models::models::parameters::edit_params::EditParams;
use geoffrey_models::models::player::Player;
use geoffrey_models::models::CommandLevel;
use std::sync::Arc;

pub struct Edit {}

impl Command for Edit {
    type Req = EditParams;
    type Resp = Location;

    fn command_name() -> String {
        "edit".to_string()
    }

    fn request_type() -> RequestType {
        RequestType::POST
    }

    fn command_level() -> CommandLevel {
        CommandLevel::REGISTERED
    }

    fn run_command(ctx: Arc<Context>, req: &Self::Req, user: Option<Player>) -> Result<Self::Resp> {
        let user = user.unwrap();

        let mut location = find_location_by_name(&ctx.db, &req.loc_name, user.id.unwrap())?;

        if let Some(new_pos) = &req.new_pos {
            location.position = *new_pos;
        }

        if let Some(new_name) = &req.new_name {
            location.name = new_name.clone();
        }

        let location = ctx.db.insert(location)?;

        load_location(&ctx.db, &location).map_err(|err| err.into())
    }
}
