use crate::commands::{Command, RequestType};
use crate::context::Context;
use crate::Result;
use geoffrey_db::helper::{find_location_by_name_type, load_location};
use geoffrey_models::models::locations::{Location, LocationDataDb, LocationType};
use geoffrey_models::models::parameters::remove_item_params::RemoveItemParameters;
use geoffrey_models::models::player::Player;
use geoffrey_models::models::response::api_error::GeoffreyAPIError;
use geoffrey_models::models::CommandLevel;
use std::sync::Arc;

pub struct RemoveItem {}

impl Command for RemoveItem {
    type Req = RemoveItemParameters;
    type Resp = Location;

    fn command_name() -> String {
        "remove_item".to_string()
    }

    fn request_type() -> RequestType {
        RequestType::POST
    }

    fn command_level() -> CommandLevel {
        CommandLevel::REGISTERED
    }

    fn run_command(ctx: Arc<Context>, req: &Self::Req, user: Option<Player>) -> Result<Self::Resp> {
        let user = user.unwrap();

        let mut shop = find_location_by_name_type(
            &ctx.db,
            &req.shop_name,
            user.id.unwrap(),
            LocationType::Shop,
        )?;

        if let LocationDataDb::Shop(shop_data) = &mut shop.loc_data {
            shop_data
                .item_listings
                .retain(|item| item.item.name.to_lowercase() != req.item_name.to_lowercase());

            let shop = ctx.db.insert(shop)?;

            load_location(&ctx.db, &shop).map_err(|err| err.into())
        } else {
            Err(GeoffreyAPIError::EntryNotFound)
        }
    }
}
